/* Frame expanding instance*/
var expander = null;
var buttonlisteners = new Array();

/* 
   parameters imported from HTML, 
   use <parameter> elements
*/
var importedParameters = null;

/* TV area */
var channels = null;
var tableStartTime = 0;
var ProgramArea = "sizu";
var ProgramWave = "vhf";
var offsetThreshold = 240;

/* table parameters */
var displaySpan = 180;
var offsetMinutes = 60;

/* layout */
var channelFormat = "";
var maxChannelLetters = 5;
var maxTitleLetters = 12;
var tableMargin = 25;

/* filename of cache described in CSV format*/
var cacheFilename = "/tmp/cachefile.csv";

var monitoringTimer = null;
var updateInterval = 60; // in seconds

/* wait timer */
var waitTimer = null;
var waitCounter = 0;

var loaderCommand;

function debug( message ) {
    var elem = document.getElementById( 'debugDiv' );
    if ( elem ) {
        elem.appendChild( document.createTextNode( message ) );
        elem.appendChild( document.createElement( 'br' ) );
    }
}

function setDefaultSelections() {
    var elem_area = document.getElementById( 'area' );
    {
        var children = elem_area.childNodes;
        for ( var i = 0; i < children.length; i++ ) {
            var option = children.item( i );
            try {
                if ( option.value == ProgramArea ) {
                    option.setAttribute( 'selected' );
                } else {
                    option.removeAttribute( 'selected' );
                }
            } catch ( e ) {}
        }
    }

    var elem_wave = document.getElementById( 'wave' );
    {
        var children = elem_wave.childNodes;
        for ( var i = 0; i < children.length; i++ ) {
            var option = children.item( i );
            try {
                if ( option.value == ProgramWave ) {
                    option.setAttribute( 'selected' );
                } else {
                    option.removeAttribute( 'selected' );
                }
            } catch ( e ) {}
        }
    }
}

function setup( event ) {
    // read parameters  from HTML
    importParameters();
    // read parameters from preferences
    ProgramArea = readPreference( "area", "tokyo" );
    ProgramWave = readPreference( "wave", "vhf" );
    setDefaultSelections();

    // layout
    var front = document.getElementById( 'front' );
    expander = new Expander( front, 700, 200 );
    expander.maxSteps = parseInt( getParameter( 'collapseSteps', 10 ) );
    expander.animationInterval = parseInt( getParameter( 'collapseWait', 33 ) );
    
    displaySpan = parseInt( getParameter( 'displaySpan', 180 ) );
    channelFormat = getParameter( 'channelFormat', "<$program-start$><$starttime$><$title$><$program-end$>" );

    // button
    var doneButton = document.getElementById( 'done' );
    if ( doneButton ) {
        createGenericButton( doneButton, "Done", flipToFront );
    }

    // preload images
    loadTimerImages();

    // set event handlers
    if ( window.widget ) {
        widget.onremove = cleanUp;
        window.onblur = function() { 
            if ( expander && expander.isCollapsed == false ) {
                toggleMode( null );
            }
        };

        cacheFilename = "/tmp/television_guide_" + widget.identifier + ".csv";
        widget.onshow = function() { updateProgram();startMonitor(); };
        widget.onhide = stopMonitor;
    }
    window.addEventListener( 'mousewheel', mouseWheel, true );
    window.addEventListener( 'keypress', keyPressed, true );
    // plugin
    try {
        if ( window.TVTableGenerator ) {
            debug( "plugin detected, loading " + cacheFilename );
            TVTableGenerator.setProgramFile( cacheFilename );
        } else {
            debug( "no plugin" );
        }
    } catch ( e ) {
        debug( e );
    }

    setButtonEvents();

    setTimeout( reload, 0 );

    // set monitor
    monitoringTimer = null;
    startMonitor();
}

function setButtonEvents() {
    //var buttonlisteners = new Array();
    {
        var b = document.getElementById( 'expandbutton' );
        buttonlisteners[ 'expand' ] = new ClickListener( b, toggleMode, "url(Images/ButtonBG2.png)", "url(Images/ButtonBG1.png)" );
    }
    {
        var b = document.getElementById( 'reloadbutton' );
        buttonlisteners[ 'reload' ] = new ClickListener( b, reload, "url(Images/ButtonBG2.png)", "url(Images/ButtonBG1.png)" );
    }
    {
        var b = document.getElementById( 'upperArrow' );
        buttonlisteners[ 'up' ] = new ClickListener( b, programTableBackward, "url(Images/Up2.png)", "url(Images/Up1.png)" );
    }
    {
        var b = document.getElementById( 'lowerArrow' );
        buttonlisteners[ 'down' ] = new ClickListener( b, programTableForward, "url(Images/Down2.png)", "url(Images/Down1.png)" );
    }
    {
        var b = document.getElementById( 'homeButton' );
        buttonlisteners[ 'home' ] = new ClickListener( b, loadChannels, "url(Images/Home2.png)", "url(Images/Home1.png)" );
    }    

}

function reload(event) {
    stopWaitTimer();
//    return;
    try {
        if ( loaderCommand ) {
            loaderCommand.cancel();
            loaderCommand = null;
            debug( "stop" );
            var button = document.getElementById( 'reloadbutton' );
            if ( button ) {
                button.innerText = "Reload";
            }
            return;
        }
    } catch ( e ) {
        debug( "??? " + e );
    }
    var pycommand = "/usr/bin/python loadtv2csv.py -area " + ProgramArea + " -o " + cacheFilename + " -wave " + ProgramWave;
    debug( pycommand );
     if ( window.widget ) {
         startWaitTimer();
         loaderCommand = widget.system( pycommand, endHandler );
         var button = document.getElementById( 'reloadbutton' );
         if ( button ) {
             button.innerText = "Stop";
         }
     }

}

function endHandler( command ) {
    //debug( "endHandler " + command );
    loaderCommand = null;
    stopWaitTimer();
    var button = document.getElementById( 'reloadbutton' );
    if ( button ) {
        button.innerText = "Reload";
    }
    var error = command.errorString;
    if ( error ) {
        showDescription( error );
        return;
    }
    if ( window.TVTableGenerator ) {
        TVTableGenerator.setProgramFile( cacheFilename );
        loadChannels();
    }
}

function stopWaitTimer() {
    debug( "stop timer" );
    if ( waitTimer == null ) return;
    clearInterval( waitTimer );
    waitTimer = null;
    var elem_wait = document.getElementById( "wait" );
    if ( elem_wait ) {
        elem_wait.style.display = "none";
    }
}

function startWaitTimer() {
    debug( "start timer" );
    if ( waitTimer ) return;
    waitCounter = 0;
    waitTimer = setInterval( __tickWaitTimer, 66 );
    var elem_wait = document.getElementById( "wait" );
    if ( elem_wait ) {
        elem_wait.style.display = "block";
    }
}

function __tickWaitTimer() {
    var elem_wait = document.getElementById( "wait" );
    //debug( "counter " + waitCounter );
    if ( elem_wait ) {
        waitCounter = ( waitCounter + 1 ) % 12;
        elem_wait.style.background = "url(Images/wait" + waitCounter + ".png)";
    }
}


function readPreference( key, defaultValue ) {
    if ( window.widget ) {
        var val_unique = widget.preferenceForKey( widget.identifier + "-" + key );
        if ( val_unique ) return val_unique;
        var val = widget.preferenceForKey( key );
        if ( val ) return val;
    }
    return defaultValue;
}

function writePreference( key, value ) {
    if ( window.widget ) {
        widget.setPreferenceForKey( value, widget.identifier + "-" + key );
        widget.setPreferenceForKey( value, key );
    }
}


function cleanUp() {
    // cancel python script
    if ( loaderCommand ) {
        loaderCommand.cancel();
    }
    // remove widget's unique preference
    if ( window.widget ) {
        widget.setPreferenceForKey( null, widget.identifier + "-area" );
        widget.setPreferenceForKey( null, widget.identifier + "-wave" );
    }
    // remove cache file
    if ( window.TVTableGenerator ) {
        window.TVTableGenerator.removeProgramFile( cacheFilename );
    }
}

function updateProgram() {
    if ( expander && expander.isCollapsed ) {
        if ( waitTimer == null ) {
            debug( "update program" );
            setCurrentPrograms();
        }
    }
}

function startMonitor() {
    if ( monitoringTimer == null ) {
        monitoringTimer = setInterval( updateProgram, 1000 * updateInterval );
    }
}

function resumeMonitor() {
    updateProgram();
    startMonitor();
}

function stopMonitor() {
    if ( monitoringTimer ) {
        clearInterval( monitoringTimer );
        monitoringTimer = null;
    }
}

function loadTimerImages() {
    for ( var i = 0; i < 12; i++ ) {
        var im = new Image();
        im.src = "Images/wait" + i + ".png";
    }
}

function reloadPrograms() {
}

function programTableForward( event ) {
    if ( tableStartTime ) {
        loadChannels( tableStartTime + offsetMinutes );
    }
}

function programTableBackward( event ) {
    if ( tableStartTime ) {
        loadChannels( tableStartTime - offsetMinutes );
    }
}

// function programTableCurrent( event ) {
//     loadChannels();
// }

function showAreaAndTime( minutes, area ) {
    if ( minutes < 0 ) minutes += 1440;
    if ( minutes >= 1440 ) minutes -= 1440;
    var hs = parseInt( minutes / 60 );
    var ms = parseInt( minutes % 60 );
    var timeString;
    if ( hs < 10 ) {
        hs = "0" + hs;
    } else {
        hs = '' + hs;
    }
    if ( ms < 10 ) {
        ms = "0" + ms;
    } else {
        ms = '' + ms;
    }
    timeString = hs + ":" + ms;

    if ( area == undefined ) {
        area = ProgramArea;
    }
    var area_name = area;
    if ( window.TVTableGenerator ) {
        area_name = TVTableGenerator.getArea();
    }

    var elem_state = document.getElementById( 'areabutton' );
    if ( elem_state ) {
        if ( area_name == "" ) {
            elem_state.innerHTML = timeString;
        } else {
            elem_state.innerHTML = area_name + "/" + timeString;
        }
        debug( area_name + ", " + timeString );
    }

    var elem_wave = document.getElementById( 'wavebutton' );
    if ( elem_wave ) {
        if ( ProgramWave == "vhf" ) {
            elem_wave.innerHTML = "VHF";
        } else if ( ProgramWave == "uhf" ) {
            elem_wave.innerHTML = "UHF";
        } else if ( ProgramWave == "bs_digital" ) {
            elem_wave.innerHTML = "BS Digital";
        } else {
            elem_wave.innerHTML = "?";
        }
    }
}

function loadChannels( startMin ) {
    var timeString = "";
    if ( startMin == undefined ) {
        var d = new Date();
        startMin = d.getHours() * 60 + d.getMinutes();
    }
    if ( startMin < 0 ) startMin += 1440;
    // set global variable
    tableStartTime = startMin;

//     {
//         var hs = parseInt( startMin / 60 );
//         var ms = parseInt( startMin % 60 );
//         if ( hs < 10 ) {
//             hs = "0" + hs;
//         } else {
//             hs = '' + hs;
//         }
//         if ( ms < 10 ) {
//             ms = "0" + ms;
//         } else {
//             ms = '' + ms;
//         }
//         timeString = hs + ":" + ms;
//     }

//     channels = new Array();
//     var area_and_time = ProgramArea;
//     if ( window.TVTableGenerator ) {
//         var num_ch = TVTableGenerator.getChannelSize();
//         for ( var i = 0; i < num_ch; i++ ) {
//             channels.push( TVTableGenerator.getChannelName( i ) );
//         }
//     }
    showAreaAndTime( startMin );

    var size = TVTableGenerator.getChannelSize();//channels.length;
    debug( "channel size = " + size );
    var width_channel = getParameter( 'channelWidth', 80 );
    var width = expander.size1.width + width_channel * size;
    if ( expander.isCollapsed ) {
        expander.size2.width = width + tableMargin;
        expander.size2.height = getParameter( 'expandHeight', expander.size2.height );
    }
    var elem_table = document.getElementById( 'programTable' );

    var children = elem_table.childNodes;
    for ( var i = children.length - 1; i >= 0; i-- ) {
        var child = children.item( i );
        try {
            if ( child.getAttribute( 'class' ) == "channel" ) {
                elem_table.removeChild( child );
            }
        } catch ( e ) {
//            debug( "removing child failed : " + e );
        }
    }

//     area_and_time = area_and_time + "/" + timeString;
//     var elem_state = document.getElementById( 'areabutton' );
//     if ( elem_state ) {
//         elem_state.innerHTML = area_and_time;
//         debug( area_and_time );
//     }

    if ( expander != null && expander.isCollapsed ) {
        debug( "current programs" );
        setCurrentPrograms();
    }

    var startMin24 = startMin;
    if ( startMin < offsetThreshold ) {
        startMin24 += 1440;
    }
    for ( var i = 0; i < size; i++ ) {
        debug( i + " / " + size );
        if ( expander.isCollapsed == false ) {
            debug( ">" + ( i * width_channel + expander.size1.width + tableMargin ) + " and " + expander.size2.width );
            if ( i * width_channel + expander.size1.width 
                 + tableMargin >= expander.size2.width ) {
                debug( "OVERFLOW!! " + i + " / " + size );
                break;
            }
        }
//         if ( i * width_channel + width_channel > expander.size2.width ) {
//             break;
//         }
//        debug( i + " / " + size );

        var elem = document.createElement( 'div' );
        try {
            var channelname = TVTableGenerator.getChannelName( i );//TVTableGenerator.getChannelName( i );
            debug( "append channel " + channelname );
            elem.setAttribute( 'class', 'channel' );
            elem.setAttribute( 'id', channelname );//channels[ i ] );
            elem.style.left = ( width_channel * i ) + "px";
            var name = document.createElement( 'span' );
            name.setAttribute( 'class', 'channelName' );
            name.appendChild( document.createTextNode( channelname ) );//s[ i ] ) );
            elem.appendChild( name );
            var progs = document.createElement( 'div' );
            progs.setAttribute( 'class', 'programs' );
            elem.appendChild( progs );
            progs.innerHTML = getProgramsWithFormat( i, channelFormat, startMin24, startMin24 + displaySpan );
            elem_table.appendChild( elem );
        } catch ( e ) {
            debug( "Error!!!! " + e );
        }
    }
}

function getProgramsWithFormat( channelnumber, format, start, end ) {
    if ( window.TVTableGenerator ) {
        //debug( "ch " + channelnumber + ", start " + start + ", end " + end );
        //getProgramListWithRange
        var pout = TVTableGenerator.getProgramListWithRange( format, channelnumber, start, end );
        //debug( pout );
        return pout;
        //return "skip plugin";
        //return "prog1<br/>prog2<br/>";
    } else {
        return "prog1<br/>prog2<br/>";
    }
}

function importParameters() {
    importedParameters = new Array();
    var params = document.getElementsByTagName( 'parameter' );
    debug( "PARAMS " + params.length );
    for ( var i = 0; i < params.length; i++ ) {
        try {
            var name = params[ i ].getAttribute( 'name' );
            var value = params[ i ].getAttribute( 'value' );
            debug( "SET " + name + " : " + value );
            importedParameters[ name ] = value;
        } catch ( e ) {
            debug( e );
        }
    }
}

function getParameter( label, defaultValue ) {
    if ( importedParameters == null ) {
        importParameters();
    }
    var params = document.getElementsByTagName( 'parameter' );
    if ( importedParameters[ label ] ) {
        return importedParameters[ label ];
    }
    return defaultValue;
}

function collapseToFlip( event ) {
    if ( expander.isMoving ) return;
    if ( expander.isCollapsed ) {
        flipToBack();
    } else {
        expander.callBack = flipToBack;
        expander.collapse();
    }
}

function flipToBack( event ) {
    debug( "flip to back" );
    changeToShortMode();
    expander.callBack = null;
    if ( window.widget ) {
        widget.prepareForTransition( "ToBack" );
    }
    var front = document.getElementById( 'front' );
    var behind = document.getElementById( 'behind' );
    var fstyle = document.defaultView.getComputedStyle( front );
    behind.style.left = fstyle.getPropertyValue( "left" ) + "px";
    behind.style.top = fstyle.getPropertyValue( "top" ) + "px";

    front.style.display = "none";
    behind.style.display = "block";
    if ( window.widget ) {
        setTimeout( "widget.performTransition()", 0 );
    }
}

function flipToFront( event ) {
    if ( window.widget ) {
        widget.prepareForTransition( "ToFront" );
    }
    var front = document.getElementById( 'front' );
    var behind = document.getElementById( 'behind' );
    front.style.display = "block";
    behind.style.display = "none";
    if ( window.widget ) {
        setTimeout( "widget.performTransition()", 0 );
    }

    var reloadFlag = false;
    var areaElem = document.getElementById( 'area' );
    if ( areaElem ) {
        var areaName = areaElem.value;
        debug( "selected area : " + areaName + " <= " + ProgramArea );
        if ( areaName != ProgramArea ) {
            ProgramArea = areaName;
            writePreference( "area", areaName );
            // reload
            reloadFlag = true;
            //reload();
        }
    }

    var waveElem = document.getElementById( 'wave' );
    if ( waveElem ) {
        var wave_name = waveElem.value;
        debug( "selected wave : " + ProgramWave + " => " + wave_name );
        if ( ProgramWave != wave_name ) {
            ProgramWave = wave_name;
            writePreference( "wave", wave_name );
            reloadFlag = true;
        }
    }

    if ( reloadFlag ) {
        // kill script
        if ( loaderCommand ) { 
            loaderCommand.cancel();
            loaderCommand = null;
        }
        setTimeout( reload, 0 );
    }
}

function toggleMode( event ) {
    if ( expander.isCollapsed ) {
        if ( window.TVTableGenerator ) {
            if ( TVTableGenerator.getChannelSize() <= 0 ) {
                return;
            }
        } else {
            return;
        }

        if ( expander ) {
            debug( "set expanding size " );
            var size = TVTableGenerator.getChannelSize();
            var width_channel = getParameter( 'channelWidth', 80 );
            var width = expander.size1.width + width_channel * size;
            expander.size2.width  = width + tableMargin;
            expander.size2.height = getParameter( 'expandHeight', expander.size2.height );
            debug( "size is ( " +expander.size2.width + ", " +  expander.size2.height + " )" );
            loadChannels();
            expander.callBack = changeToTableMode;
        }
    } else {
        var tables = document.getElementById( "programTable" );
        tables.style.display = "none";
        expander.callBack = changeToShortMode;
    }
    if ( expander ) {
        expander.toggle(event);
    }
}

function changeToTableMode() {
    expander.callBack = null;
    var description = document.getElementById( 'description' );
    description.innerHTML = "";

    var tables = document.getElementById( "programTable" );
    tables.style.display = "block";
}

function changeToShortMode() {
    expander.callBack = null;
    var description = document.getElementById( 'description' );
    //description.innerHTML = "SHORT";
    setCurrentPrograms();
}

function setCurrentPrograms() {
    var contents = "";
    if ( window.TVTableGenerator ) {
        var date = new Date();
        var minutes = date.getHours() * 60 + date.getMinutes();
        if ( minutes < offsetThreshold ) {
            minutes += 1440;
        }
        contents = TVTableGenerator.getCurrentPrograms( minutes );
        showAreaAndTime( minutes );
        //debug( "set current programs : " + minutes );
    } else {
        return;
    }

    var rows = contents.split( "\n" );
    var table = document.createElement( "table" );
    for ( var i = 0; i < rows.length; i++ ) {
        var columns = rows[ i ].split( ',' );
        if ( columns.length < 2 ) continue;
        var row = document.createElement( 'tr' );
        {
            var datum = document.createElement( 'td' );
            datum.setAttribute( 'class', 'cncolumn' );
            var chname = columns[ 0 ];
            if ( chname.length > maxChannelLetters ) {
                chname = chname.substring( 0, maxChannelLetters );// + "..";
            }
            datum.appendChild( document.createTextNode( chname ) );
            row.appendChild( datum );
        }
        {
            var datum = document.createElement( 'td' );
            var title = columns[ 1 ];
            if ( title.length > maxTitleLetters ) {
                title = title.substring( 0, maxTitleLetters );// + "..";
            }
            datum.appendChild( document.createTextNode( title ) );
            row.appendChild( datum );
        }
        table.appendChild( row );
    }

    var description = document.getElementById( 'description' );
    if ( description ) {
        description.innerHTML = "";
        description.appendChild( table );
    }
}

function showProgramDescription( elem ) {
    var html = elem.innerHTML;
    var desc = elem.description;
    if ( desc ) {
        html = html + "<br/>" + desc;
    }
    showDescription( html );
}

function showDescription( text ) {
    var description = document.getElementById( 'description' );
    if ( description ) {
        description.innerHTML = text;
    }
}

function mouseWheel( event ) {
    var delta = event.wheelDelta;
    debug( "wheel " + event.delta );
    var clicker = null;
    if ( delta > 0 ) {
        var clicker = buttonlisteners[ 'up' ];
    } else {
        var clicker = buttonlisteners[ 'down' ];
    }
    if ( clicker ) clicker.click();
}

function keyPressed( event ) {
    debug( "keypressed " + event.keyCode );
    var flag_shift = event.shiftKey;
    var flag_alt = event.altKey;
    var flag_ctrl = event.ctrlKey;
    var flag_meta = event.metaKey;
    var code = event.keyCode;
    if ( flag_shift || flag_alt || flag_ctrl || flag_meta ) return;
    if ( code == 63232 ) {
        var clicker = buttonlisteners[ 'up' ];
        if ( clicker ) clicker.click();
    } else if ( code == 63233 ) {
        var clicker = buttonlisteners[ 'down' ];
        if ( clicker ) clicker.click();
    }
    event.stopPropagation();
    event.preventDefault();
}
